function [OutCellArray,Outconfig]=BadDataRemoval(InCellArray, Inconfig)
%This module provides a GUI for selectively removing bad or invalid data
%points from a map, and then interpolates or extrapolates the missing data
%back.
%
%Inputs: Map
%Outputs: Map
%
%Author: Benjamin Lewis
%Email: blewis@email.arizona.edu
%
% Version history:
% 7/25/10 - Version 1
% 8/20/11 - version 1.1 by Dae Wook Kim letter2dwk@hotmail.com

Outconfig=Inconfig;

OutCellArray{1}=InCellArray{1};

dmap=InCellArray{1}.data;


% dmapforplot=dmap;
% 
% dmin=min(dmap(~isnan(dmap))); %Value used for plotting
% 
% if dmin>0, dmin=.6*dmin; else dmin=dmin/.6; end
% 
% dmapforplot(isnan(dmap))=dmin;  %Construct used to create valid plot, imagesc doesn't accept NaN


%fig=figure('Visible', 'off', 'units', 'pixels','toolbar','none','MenuBar','none');
fig=figure('Visible', 'off', 'units', 'pixels', 'WindowButtonDownFcn',@ButtonDown, ...
    'WindowKeyPressFcn',@KeyPress);
movegui(fig,'center');
imagesc(dmap,'alphadata',~isnan(dmap));
set(fig,'pointer','crosshair');
set(fig,'Visible','on');
figure(fig);


hlp=helpdlg('Click on a point to remove it from the data.  When finished, close the figure window.','Bad Data Removal');


badmap=zeros(size(dmap));

    function ButtonDown(src,evnt)
        %Code stolen from ginput (which does not provide all
        %functionality needed).
        scrn_pt = get(0, 'PointerLocation');
        loc = get(fig, 'Position');
        pt = [scrn_pt(1) - loc(1) + 1, scrn_pt(2) - loc(2) + 1];
        set(fig,'CurrentPoint',pt);
        pt = get(gca, 'CurrentPoint');
        %End stolen code
        if pt(1)>0&&pt(2)>0&&pt(1)<size(dmap,1)&&pt(2)<size(dmap,2)
            y=round(pt(1,1));
            x=round(pt(1,2));
            badmap(x,y)=1;
            dmap(x,y)=NaN;            
            imagesc(dmap,'alphadata',~isnan(dmap));
        end
    end

    function KeyPress(src,evnt)
        close(fig);
    end

waitfor(fig);

if ishandle(hlp), close(hlp); end

badmap(isnan(InCellArray{1}.data))=0; %clear eronious clicks

if max(badmap(:))>0 %user actually selected some points    
    tempdmap=dmap;
    x=1:size(tempdmap,1);
    y=1:size(tempdmap,2);
    [x,y]=meshgrid(x,y);   
    
    %Good Interpolation (where we have support)
    InterpMap=TriScatteredInterp(x(~isnan(tempdmap)), y(~isnan(tempdmap)), tempdmap(~isnan(tempdmap)));
    tempdmap=InterpMap(x,y);
    
    if isempty(tempdmap), OutCellArray={}; errordlg('Insufficient Data for Reconstruction.'); ...
            return; end %Results from insufficient valid data
    
    %Fill in pixels we've been able to interpolate
    for ii=1:size(dmap,1), for jj=1:size(dmap,2), if badmap(ii,jj)&&~isnan(tempdmap(ii,jj)),
                dmap(ii,jj)=tempdmap(ii,jj); end, end, end
    
    badmap(~isnan(tempdmap))=0; %Clear bad pixels we've been able to fit from bad pixel map
    tempdmap=dmap;
    
    %Crappy interpolation (outside convex hull)
    if any(badmap(:)==1)
        tempdmap=griddata(x(~isnan(tempdmap)), y(~isnan(tempdmap)), tempdmap(~isnan(tempdmap)), x, y, 'v4');        
        
        %Fill in pixels we've been able to interpolate
        for ii=1:size(dmap,1), for jj=1:size(dmap,2), if badmap(ii,jj)&&~isnan(tempdmap(ii,jj)),
                    dmap(ii,jj)=tempdmap(ii,jj); end, end, end
        
        badmap(~isnan(tempdmap))=0; %Clear bad pixels we've been able to fit from bad pixel map
        tempdmap=dmap;
    end
        
    %Now we just have to dilate and hope for the best (horrible results to
    %be expected)
    if any(badmap(:)==1)
        %Dilation
        stats=regionprops(badmap,'BoundingBox');
        
        xmax=max(stats.BoundingBox(:,3));
        
        ymax=max(stats.BoundingBox(:,4));
        
        strsize=sqrt(xmax^2+ymax^2)/2; %Smallest possible strel object that ensures dilation
        
        strsize=ceil(strsize);
        
        str=strel('ball', strsize, 1/strsize, 0);
        
        tempdmap(isnan(tempdmap))=-1*realmax;
        tempdmap=imdilate(tempdmap, str);
        
        %Fill remaining bad pixels with dilated data
        for ii=1:size(dmap,1), for jj=1:size(dmap,2), if badmap(ii,jj),
                    dmap(ii,jj)=tempdmap(ii,jj); end, end, end
    end
    
    OutCellArray{1}.data=dmap;
else
    errordlg('No points selected! No data will be returned','Bad Data Removal','modal');
    OutCellArray={};
end


end

